/*

Set Scale v1.2.1
Matt Gorner, 4 March 2008

This lscript contains some code from freely available lscripts by Carl Merritt and Umezawa Masayuki (Pictrix)
I'd like to thank these (and other) lscript authors for making their code available for others to learn from.


Info:

When importing 2D drawings / plans for modelling as either DXF or Illustrator format (or even imported 3D
models) you nearly always have to scale them to their correct size.

This means measuring two points, finding the scale ratio on a calculator, then scaling the geometry.

Although it's a simple process, it could benefit from automation, especially if you import a lot of data into LightWave
from other packages (2D or 3D).

This script simply automates those steps.

Usage:

1) Pick two points that you know the real dimensions for (you can select more than one point, but the script takes the
	first and last point selected making it easier to select points on 2D geometry in one axis)

2) Run the plugin and enter the 'real' value of the distance

3) Select whether you want to measure the LW measured distance in true XYZ space, or only the difference in either X, Y, Z axis

4) Choose whether to scale from the world origin (0, 0, 0) or the objects bounding box centre

5) Click 'Ok' to scale your geometry.

That's it! The plugin will then scale _everything_ on _all_ active layers to make the distance between the first and last selected
points the same as the 'real' distance specified.

Hope you find it useful!

Cheers
Matt

*/

@warnings
@version 2.3
@name Set Scale

gadget_source;
selected_points;

main
{

	// User interface layout variables

	layout_x						= -10;
	layout_y						= 15;
	layout_width				= 350;
	layout_height				= 19;

	slot_offset					= 19;
	spacer						= 3;
	text_offset					= 150;
	number_of_gadgets			= 10;

	req_width					= 355;
	req_height					= ((number_of_gadgets * (layout_height + spacer)) + (2 * layout_height)) - spacer;

	// Pull previously used values for requester
	scale_centre				= recall("matt_scale_centre", 2);
	measure_axis				= recall("matt_measure_axis", 1);
	scale_x						= recall("matt_scale_x", 1);
	scale_y						= recall("matt_scale_y", 1);
	scale_z						= recall("matt_scale_z", 1);

	distance						= calculate_distance(measure_axis);

	// Setup interface . . .
 	reqbegin("Set Scale . . .");
	reqsize(req_width, req_height);
	
	// Layout gadgets

	row_number = 0;

	// Setup gadgets
	// Initialise a gadget . . .
	gadget_points = ctlstring("# of points selected :", string(selected_points) );
	// Lay it out . . .
	ctlposition(gadget_points , layout_x, layout_y + (row_number * slot_offset), layout_width, layout_height, text_offset);
	
	row_number ++ ;

	gadget_source = ctldistance("LW Measured Distance :", distance );
	ctlposition(gadget_source, layout_x, layout_y + (row_number * slot_offset) + (row_number * spacer), layout_width, layout_height, text_offset);
	
	row_number ++ ;

	gadget_real = ctldistance("Real Distance :", 0);
	ctlposition(gadget_real, layout_x, layout_y + (row_number * slot_offset) + (row_number * spacer), layout_width, layout_height, text_offset);
	
	row_number ++ ;

	gadget_measurement_axis = ctlchoice("Measure Axis :", measure_axis, @"XYZ","Delta X","Delta Y", "Delta Z"@, false);
	ctlposition(gadget_measurement_axis, layout_x, layout_y + (row_number * slot_offset) + (row_number * spacer), layout_width, layout_height, text_offset);
	
	row_number ++ ;

	gadget_seperator = ctlsep(0, 235);
	ctlposition(gadget_seperator, -2, layout_y + (row_number * slot_offset) + (row_number * spacer) + (layout_height / 2), req_width + 4);
	
	row_number ++ ;

	gadget_bbscale = ctlchoice("Scale Centre :", scale_centre, @"Origin (0, 0, 0)","Bounding Box"@, false);
	ctlposition(gadget_bbscale, layout_x, layout_y + (row_number * slot_offset) + (row_number * spacer), layout_width, layout_height, text_offset);

	row_number ++ ;

	gadget_scalex = ctlcheckbox("Scale X Axis", scale_x);
	ctlposition(gadget_scalex, layout_x, layout_y + (row_number * slot_offset) + (row_number * spacer), layout_width, layout_height, text_offset);

	row_number ++ ;

	gadget_scaley = ctlcheckbox("Scale Y Axis", scale_y);
	ctlposition(gadget_scaley, layout_x, layout_y + (row_number * slot_offset) + (row_number * spacer), layout_width, layout_height, text_offset);

	row_number ++ ;

	gadget_scalez = ctlcheckbox("Scale Z Axis", scale_z);
	ctlposition(gadget_scalez, layout_x, layout_y + (row_number * slot_offset) + (row_number * spacer), layout_width, layout_height, text_offset);

	ctlrefresh( gadget_measurement_axis, "refresh_lw_measured_distance_gadget" );

	// Show interface and gadgets (return back if it can't do it)
	return if !reqpost();

		// Grab user values
		source_distance	= getvalue(gadget_source);
		real_distance		= getvalue(gadget_real);
		measure_axis		= getvalue(gadget_measurement_axis);
		scale_centre		= getvalue(gadget_bbscale);
		scale_x				= getvalue(gadget_scalex);
		scale_y				= getvalue(gadget_scaley);
		scale_z				= getvalue(gadget_scalez);

		// store our variables for the next time the script is run (within one LW session, not if you quit)
		store("matt_scale_centre", scale_centre);
		store("matt_measure_axis", measure_axis);
		store("matt_scale_x", scale_x);
		store("matt_scale_y", scale_y);
		store("matt_scale_z", scale_z);

	// Close requester
	reqend();

	// Do stuff . . .

	// Don't want to do any division by 0!
	if( source_distance > 0 )
	{
		// Clear the selected points prior to scaling otherwise only they will be scaled, we want all the geometry!
		selpoint(CLEAR);

		// Now find bounding box centre . . .
		bounding_box_centre = center(boundingbox());

		// Pull out the X,Y,Z coords from the 'bounding_box_centre' variable
		bbx = number(bounding_box_centre.x);
		bby = number(bounding_box_centre.y);
		bbz = number(bounding_box_centre.z);

		// Work out scale ratio . . .
		scale_ratio = abs( ( real_distance / source_distance ) ) ;

		// Change ratio if told not to scale in a specific axis
		if (scale_x)
		{
			scale_ratio_x = scale_ratio;
		}
		else
		{
			scale_ratio_x = 1;		
		}

		if (scale_y)
		{
			scale_ratio_y = scale_ratio;
		}
		else
		{
			scale_ratio_y = 1;		
		}

		if (scale_z)
		{
			scale_ratio_z = scale_ratio;
		}
		else
		{
			scale_ratio_z = 1;		
		}

		// Which centre are we scaling to?
		if( scale_centre == 2 )
		{
			// Scale geometry using bounding box centre
			scale( <scale_ratio_x, scale_ratio_y , scale_ratio_z>, <bbx,bby,bbz>);
			
			// Feedback
			info( "LW Measured Distance : ", source_distance,"   Real Distance : ", real_distance, "   Scale Ratio : ", scale_ratio );
		}
		else
		{
			// Scale geometry using 0,0,0 as centre
			scale( <scale_ratio_x, scale_ratio_y , scale_ratio_z>, <0,0,0>);
			
			// Feedback
			info( "LW Measured Distance : ", source_distance,"   Real Distance : ", real_distance, "   Scale Ratio : ", scale_ratio );
		}

	}
	else
	{
		// User tried to divide by 0
		error("The 'Measured Distance' cannot be 0, select two points that have some distance");
	}
	
	// Finished!
	return;
}

// Gadget callback procedure ...

refresh_lw_measured_distance_gadget: value
{
	// Calulate distance based on 'Measure Axis' preference
	distance = calculate_distance(value);

	// Update the 'LW Measured Distance' gadget to reflect user preference
	setvalue(gadget_source, distance);
	
	return;
}

calculate_distance: measure_axis
{

	// Grab user selection (I wanted to use selmode( DIRECT ) which only returns REAL selected
	// points, and not using the 'if nothing is selected everything is selected' philosophy that USER
	// mode returns. However selpoint(CLEAR) only seems to work in USER mode.
	selmode( USER );

	// Get total number of points selected
	selected_points = editbegin();

	// No geometry at all?
	if (selected_points == 0)
	{
		error("There is no geometry to work with");
	}
	
	// FIRST points coords
	x1 = pointinfo(points[1]).x;
	y1 = pointinfo(points[1]).y;
	z1 = pointinfo(points[1]).z;

	// LAST points coords
	x2 = pointinfo(points[selected_points]).x;
	y2 = pointinfo(points[selected_points]).y;
	z2 = pointinfo(points[selected_points]).z;

	// Have everything we need so end edit mode
	editend();
	
	// Get difference between points

	difference_x = (x2 - x1);
	difference_y = (y2 - y1);
	difference_z = (z2 - z1);

	// Work out distance based on 'Measure Axis' preference

	if( measure_axis == 1 )  // XYZ
	{
		// Work out XYZ distance using Pythagoras theorem
		distance = sqrt((difference_x * difference_x) + (difference_y * difference_y) + (difference_z*difference_z));
	}
	else if( measure_axis == 2 ) // X
	{
		// Work out difference in X axis, and make sure it's not a negative number
		distance = abs((x2 - x1));
	}
	else if( measure_axis == 3 ) // Y
	{
		// Work out difference in Y axis, and make sure it's not a negative number
		distance = abs((y2 - y1));
	}
	else if( measure_axis == 4 ) // Z
	{
		// Work out difference in Z axis, and make sure it's not a negative number
		distance = abs((z2 - z1));
	}

	return( distance );
}
